/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.yourorghere.module5;

import java.awt.datatransfer.Transferable;
import java.util.*;
import java.io.*;
import java.beans.*;

import javax.swing.text.JTextComponent;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import org.openide.loaders.*;
import org.openide.filesystems.*;
import org.openide.nodes.*;
import org.openide.xml.XMLUtil;
import org.openide.actions.*;
import org.openide.util.Utilities;
import org.openide.util.NbBundle;
import org.openide.ErrorManager;
import org.openide.text.ActiveEditorDrop;
import org.openide.util.Lookup;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;


/**
 * DataObject for palette item file. It reads the file and creates PaletteItem
 * and node from it.
 *
 * @author Tomas Pavek
 */

class PaletteItemDataObject extends MultiDataObject {

    static final String XML_ROOT = "palette_item"; // NOI18N
    static final String ATTR_VERSION = "version"; // NOI18N
    static final String TAG_COMPONENT = "component"; // NOI18N
    static final String ATTR_CLASSNAME = "classname"; // NOI18N
    static final String ATTR_TYPE = "type"; // NOI18N
//    static final String ATTR_IS_CONTAINER = "is-container"; // NOI18N
    static final String TAG_CLASSPATH = "classpath"; // NOI18N
    static final String TAG_RESOURCE= "resource"; // NOI18N
    static final String ATTR_NAME = "name"; // NOI18N
    static final String TAG_DESCRIPTION = "description"; // NOI18N
    static final String ATTR_BUNDLE = "localizing-bundle"; // NOI18N
    static final String ATTR_DISPLAY_NAME_KEY = "display-name-key"; // NOI18N
    static final String ATTR_TOOLTIP_KEY = "tooltip-key"; // NOI18N
    static final String TAG_ICON16 = "icon16"; // NOI18N
    static final String ATTR_URL = "urlvalue"; // NOI18N
    static final String TAG_ICON32 = "icon32"; // NOI18N
    // component types: "visual", "menu", "layout", "border"

    private static final Node.PropertySet[] NO_PROPERTIES = new Node.PropertySet[0];

    private boolean fileLoaded; // at least tried to load

    private PaletteItem paletteItem;

    // some raw data read from the file (other passed to PaletteItem)
    private String displayName_key;
    private String tooltip_key;
    private String bundleName;
    private String icon16URL;
    private String icon32URL;

    // resolved data (derived from raw data)
    String displayName;
    String tooltip;
    java.awt.Image icon16;
    java.awt.Image icon32;

    // --------

    PaletteItemDataObject(FileObject fo, MultiFileLoader loader)
        throws DataObjectExistsException
    {
        super(fo, loader);
    }

    boolean isFileRead() {
        return fileLoaded;
    }

    boolean isItemValid() {
        return paletteItem != null;
    }

    void reloadFile() {
        if (paletteItem != null) {
            paletteItem.reset(); // resets resolved data (but not raw data)
        }

        displayName = null;
        tooltip = null;
        icon16 = null;
        icon32 = null;

        displayName_key = null;
        tooltip_key = null;
        bundleName = null;
        icon16URL = null;
        icon32URL = null;

        loadFile();
    }

    // ------

    @Override
    public Node createNodeDelegate() {
        return new ItemNode();
    }

    @Override
    public <T extends Node.Cookie> T getCookie(Class<T> cookieClass) {
        if (PaletteItem.class.equals(cookieClass)) {
            if (!fileLoaded)
                loadFile();
            return cookieClass.cast(paletteItem);
        }
        return super.getCookie(cookieClass);
    }

    // -------

    private void loadFile() {
        fileLoaded = true;
        PaletteItem item = paletteItem;
        if (item == null)
            item = new PaletteItem(this);

        FileObject file = getPrimaryFile();
        if (file.getSize() == 0L) { // item file is empty
            // just derive the component class name from the file name
            paletteItem = item;
            return;
        }
        
        // parse the XML file
//        try {
//            XMLReader reader = XMLUtil.createXMLReader();
//            PaletteItemHandler handler = new PaletteItemHandler();
//            reader.setContentHandler(handler);
//            InputSource input = new InputSource(getPrimaryFile().getURL().toExternalForm());
//            reader.parse(input);
//            // TODO report errors, validate using DTD?
//            
//            item.setComponentExplicitType(handler.componentExplicitType);
//            if (handler.componentClassName != null || displayName_key != null) {
//                item.setComponentClassSource(new ClassSource(handler.componentClassName, handler.entries));
//                paletteItem = item;
//            }
//        } catch (SAXException saxex) {
//            ErrorManager.getDefault().notify(ErrorManager.INFORMATIONAL, saxex);
//        } catch (IOException ioex) {
//            ErrorManager.getDefault().notify(ErrorManager.INFORMATIONAL, ioex);
//        }
    }


    // -------

    /** DataLoader for the palette item files. */
    public static final class PaletteItemDataLoader extends UniFileLoader {

        static final String ITEM_EXT = "my_palette_item"; // NOI18N
        static final String ITEM_MIME = "text/x-my-palette-item"; // NOI18N

        PaletteItemDataLoader() {
            super("org.yourorghere.module5.PaletteItemDataObject"); // NOI18N

            ExtensionList ext = new ExtensionList();
            ext.addMimeType(ITEM_MIME);
            setExtensions(ext);
        }
        
        /** Gets default display name. Overides superclass method. */
        @Override
        protected String defaultDisplayName() {
            return "My Palette Item Loader";
        }
        

        protected MultiDataObject createMultiObject(FileObject primaryFile)
            throws DataObjectExistsException, IOException
        {
            return new PaletteItemDataObject(primaryFile, this);
        }
    }
    
    // --------

    /** Node representing the palette item (node delegate for the DataObject). */
    class ItemNode extends DataNode {

        ItemNode() {
            this(new InstanceContent(), PaletteItemDataObject.this);
        }
        
        private ItemNode( InstanceContent ic, PaletteItemDataObject dob ) {
            super( dob, Children.LEAF, new ProxyLookup( new Lookup[] { /*dob.getLookup(),*/ new AbstractLookup(ic) } ));
            ic.add( new MyDropImpl() );
            ic.add( dob );
        }

        @Override
        public String getDisplayName() {
            if (!fileLoaded)
                loadFile();

            if (displayName == null) {
                displayName = super.getDisplayName();
            }
            return displayName;
        }

        @Override
        public String getShortDescription() {
            if (!fileLoaded)
                loadFile();

            if (tooltip == null) {
                tooltip = super.getShortDescription();
            }
            return tooltip;
        }

        @Override
        public boolean canRename() {
            return false;
        }

        @Override
        public java.awt.Image getIcon(int type) {
            if (!fileLoaded)
                loadFile();

            if (type == BeanInfo.ICON_COLOR_32x32
                    || type == BeanInfo.ICON_MONO_32x32)
            {
                if (icon32 == null) {
                        icon32 = Utilities.loadImage("org/netbeans/modules/form/resources/palette/unknown32.gif"); // NOI18N
                }
                return icon32;
            }
            else { // small icon by default
                if (icon16 == null) {
                        icon16 = Utilities.loadImage("org/netbeans/modules/form/resources/palette/unknown.gif"); // NOI18N
                }
                return icon16;
            }
            // TODO badged icon for invalid item?
        }

        // TODO properties
        @Override
        public Node.PropertySet[] getPropertySets() {
            return NO_PROPERTIES;
        }

        public Transferable clipboardCopy() throws IOException {

            ExTransferable t = ExTransferable.create( super.clipboardCopy() );

            Lookup lookup = getLookup();
            ActiveEditorDrop drop = (ActiveEditorDrop) lookup.lookup(ActiveEditorDrop.class);
            ActiveEditorDropTransferable s = new ActiveEditorDropTransferable(drop);
            t.put(s);

            return t;
        }

        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

    }

    private static class ActiveEditorDropTransferable extends ExTransferable.Single {

        private ActiveEditorDrop drop;

        ActiveEditorDropTransferable(ActiveEditorDrop drop) {
            super(ActiveEditorDrop.FLAVOR);

            this.drop = drop;
        }

        public Object getData () {
            return drop;
        }

    }

    static public class MyDropImpl implements ActiveEditorDrop {

        public boolean handleTransfer(JTextComponent targetComponent) {
            System.out.println("Should be printed when dropping a snippet in a php file=" + this);
            return true;
        }
    }
    
}
